<?php

namespace FiloBlu\Flow\Helper;

class System
{
    /**
     * @var array
     */
    protected $_stoppableIndexers =
        [
            //solo per category
            //'catalog_category_product',
            'catalog_product_category',
            'catalog_product_price',
            'catalog_product_attribute',
            'cataloginventory_stock',
            'catalogrule_rule',
            'catalogrule_product',
            'catalogsearch_fulltext',
            'design_config_grid',
            'catalog_category_flat',
            'catalog_product_flat',
            'blackbird_contenttype_fulltext',
            'mana_filter_attribute'
        ];
    /**
     * @var
     */
    protected $_indexerCollection;

    /**
     * @var
     */
    protected $_productMetadata;

    /**
     * @var
     */
    protected $_resourceConnection;
    /**
     * System constructor.
     * @param \Magento\Indexer\Model\Indexer\CollectionFactory|null $collectionFactory
     */
    public function __construct(
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Indexer\Model\Indexer\CollectionFactory $collectionFactory,
        \Magento\Framework\App\ResourceConnection $resourceConnection
    ) {

        $this->_productMetadata = $productMetadata;
        $this->_resourceConnection = $resourceConnection;

        if ($collectionFactory !== null) {
            $this->_indexerCollection = $collectionFactory->create();
        }
    }

    /**
     *
     */
    public function dropTriggers()
    {
        $connection = $this->_resourceConnection->getConnection();

        $triggers = $connection->fetchAll('SHOW TRIGGERS',[], \Zend_Db::FETCH_OBJ);

        foreach($triggers as $trigger)
        {
            $connection->query("DROP TRIGGER {$trigger->Trigger}");
        }
    }

    /**
     *
     */
    public function disableIndexers()
    {
        $indexers = $this->_indexerCollection;

        if ($indexers === null) {
            return;
        }

        foreach ($indexers->getItems() as $indexer) {
            try {
                if (!$indexer->isScheduled()) {
                    continue;
                }

                $indexer->setScheduled(false);

            } catch (\Exception $e) {

            }
        }

    }

    /**
     *
     */
    public function enableIndexers()
    {
        $indexers = $this->_indexerCollection;

        if ($indexers === null) {
            return;
        }

        foreach ($indexers->getItems() as $indexer) {
            try {
                $indexer->setScheduled(true);
            } catch (\Exception $e) {

            }
        }
    }

    /**
     * @return string
     */
    protected function getEntityField()
    {
        $edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise' || $edition == 'B2B') ? 'row_id' : 'entity_id';
    }

}
