<?php
namespace FiloBlu\Flow\Model\Channel\In;

class Price extends AbstractModel {

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel\Config $channelConfig
    ) {
        parent::__construct($loggerProvider, $channelConfig);
        $this->_logger = $loggerProvider->getLogger();
    }

    public function load($channelname) {
        $this->init($channelname);
    }

    public function insertData($data) {

        $flow = $this->getFile()->getData('flow');
        $model =  \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model' . "\\" . $flow);

        if (!$model) {
            return false;
        }

        $parser = $this->getParser();
        $check_mapping_count = $parser->check_mapping_count;

        $model->setData('meta_file',$this->getFile()->getId());
        $codice_articolo = $data['sku'];
        $model->setData("sku", $codice_articolo);

        // No check mapping count, working as CSV headers -> table columns
        if(!$check_mapping_count || $this->map->isDirectInsert()){
            // No map is checked here
            $this->directInsert($data,$model);
            // Do not go further
            return true;
        }

        foreach($data as $field => $value){

            if(floatval($value) <= 0 ) continue;
            $fieldMap = $this->map->get($field);

            if(!isset($fieldMap["skip"])) {

                //devo inserire una riga per ogni website/prezzo
                if ($fieldMap['type'] == 'price'){

                    if(@is_array($fieldMap['website'])){
                        $websites = $fieldMap['website'];

                        foreach($websites as $website){

                            try{

                                $currency = $fieldMap['currency'];
                                $model->setId(null);
                                $model->setData("prezzo", floatval($value));
                                $model->setData("paese", $website);
                                $model->setData("valuta", $currency);
                                $model->setData("meta_processed", 0);
                                $model->setData("meta_insert_time", date("Y-m-d H:i:s", time()));
                                $model->save();

                            } catch(\Exception $e) {
                                $this->_logger->info('Exception! '. $e->getMessage() . PHP_EOL);
                            }

                        }
                    }
                    elseif (@isset($fieldMap['website']) && @$fieldMap['website'] != '') {

                        try{

                            $currency = $fieldMap['currency'];
                            $website = $fieldMap['website'];
                            $model->setId(null);
                            $model->setData("prezzo", floatval($value));
                            $model->setData("paese", $website);
                            $model->setData("valuta", $currency);
                            $model->setData("meta_processed", 0);
                            $model->setData("meta_insert_time", date("Y-m-d H:i:s", time()));
                            $model->save();

                            // $this->_logger->info('Saved!! '. print_r($model->getData(), true) . PHP_EOL);

                        } catch(\Exception $e) {
                            $this->_logger->info('Exception! '. $e->getMessage() . PHP_EOL);
                        }
                    }

                    else {

                        $websites_list = explode(",",$data["paese"]);

                        try{

                            if (@empty($websites_list)) {
                                $this->_logger->info(' no website found for : ' . $data["sku"] . PHP_EOL);
                                return false;
                            }

                            foreach($websites_list as $website){

                                $model->setId(null);
                                $model->setData("prezzo", floatval($value));
                                $model->setData("paese", $website);
                                $model->setData("valuta", $data["valuta"]);
                                $model->setData("meta_processed", 0);
                                $model->setData("meta_insert_time", date("Y-m-d H:i:s", time()));
                                $model->save();

                            }

                            // $this->_logger->info('Saved!! '. print_r($model->getData(), true) . PHP_EOL);

                        } catch(\Exception $e) {
                            $this->_logger->info('Exception! '. $e->getMessage() . PHP_EOL);
                        }

                    }
                }
            }
        }

    }

    private function directInsert($data,$model){

        try{
            foreach ($data as $field => $value) {
                $model->setData($field, $value);
            }
            $model->setData("meta_processed", 0);
            $model->setData("meta_insert_time", date("Y-m-d H:i:s", time()));
            $model->save();
        } catch(\Exception $e) {
            $this->_logger->info('Exception! '. $e->getMessage() . PHP_EOL);
        }
    }

}