<?php
namespace FiloBlu\Flow\Model;

/**
 * Class Map
 * @package FiloBlu\Flow\Model
 */
class Map{
    /**
     * @var bool
     */
    public $initialized = false;
    /**
     * @var bool
     */
    protected $_isDirectInsert = false;
    /**
     * @var bool
     */
    protected $_expandConfigurables = false;
    /**
     * @var string
     */
    public $file = 'map.json';
    /**
     * @var array
     */
    protected $mappedData = [];
    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    /**
     * Map constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
    )
    {

        $this->_logger = $loggerProvider->getLogger();

    }

    /**
     * @param $channel
     */
    public function load($channel) {

        $model_channel = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Channel');
        $model_channel->load($channel,'name');



        if($model_channel){

            try {
                $json_content = str_replace(array("\n", "\t", "\r"), '', $model_channel->getData('data'));
                $mapped_config = json_decode($json_content,true);
                $this->mappedData = $mapped_config['map'];
                // Load custom properties in map object
                $this->_loadGeneralBehavior();
                $this->initialized = true;
                $this->_logger->info("Config "."Loaded map json file");
            }catch(\Exception $e){
                $this->_logger->info("Config "."Fail to load map json file");
            }
        }else{
            $this->_logger->info("Config","File for channel data not exist for json map");
        }

    }

    /**
     * Load custom properties in map object
     */
    protected function _loadGeneralBehavior(){
        foreach($this->mappedData as $field){
            foreach($field as $k => $v){
                if($k === 'direct_insert' && $v == 1){
                    $this->_isDirectInsert = true;
                    break;
                }
                if($k === 'expand_configurables' && $v == 1){
                    $this->_expandConfigurables = true;
                    break;
                }
            }
        }
    }

    public function get($field){
        if(isset($this->mappedData[$field])){
            return $this->mappedData[$field];
        }
        return null;
    }

    public function getFields(){
        return array_keys($this->mappedData);
    }

    public function getFullMappedData(){
        return $this->mappedData;
    }

    /**
     * Check if parser need to insert record directly, without any kind of processing
     *
     * @return bool
     */
    public function isDirectInsert(){
        return $this->_isDirectInsert;
    }

    /**
     * Check if parser need to insert record directly, without any kind of processing
     *
     * @return bool
     */
    public function expandConfigurables(){
        return $this->_expandConfigurables;
    }

    /**
     * @param array $mappedData
     */
    public function setFullMappedData($mappedData)
    {
        $this->mappedData = $mappedData;
    }

    public function checkData($data){

        if(count($data) == count($this->mappedData)){
            return true;
        }
        return false;
    }

}
